//===========================================================================
// @(#) $Name:  $
// @(#) $Id: DwmWWWGalleryDropUpload.hh,v 1.4 2015/05/02 08:35:41 dwm Exp $
//===========================================================================
//  Copyright (c) Daniel W. McRobb 2015
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//  3. The names of the authors and copyright holders may not be used to
//     endorse or promote products derived from this software without
//     specific prior written permission.
//
//  IN NO EVENT SHALL DANIEL W. MCROBB BE LIABLE TO ANY PARTY FOR
//  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//  INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF DANIEL W. MCROBB HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
//
//  THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND
//  DANIEL W. MCROBB HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
//  UPDATES, ENHANCEMENTS, OR MODIFICATIONS. DANIEL W. MCROBB MAKES NO
//  REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER
//  IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE,
//  OR THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT,
//  TRADEMARK OR OTHER RIGHTS.
//===========================================================================

//---------------------------------------------------------------------------
//!  \file DwmWWWGalleryDropUpload.hh
//!  \brief DropUpload class definition.
//---------------------------------------------------------------------------

#ifndef _DWMWWWGALLERYDROPUPLOAD_HH_
#define _DWMWWWGALLERYDROPUPLOAD_HH_

#include <set>
#include <string>

#include <WContainerWidget>
#include <WLabel>

#include "DwmWWWGalleryUploadResource.hh"

namespace Dwm {

  namespace WWW {

    namespace Gallery {

      //----------------------------------------------------------------------
      //!  A widget that allows drag and drop upload of files.
      //----------------------------------------------------------------------
      class DropUpload
        : public Wt::WLabel
      {
      public:
        //--------------------------------------------------------------------
        //!  Default constructor.
        //--------------------------------------------------------------------
        DropUpload(Wt::WContainerWidget *parent = 0);
        
        //--------------------------------------------------------------------
        //!  Construct with label text.
        //--------------------------------------------------------------------
        DropUpload(const Wt::WString & text, Wt::WContainerWidget *parent = 0);
        
        //--------------------------------------------------------------------
        //!  Construct with an image.
        //--------------------------------------------------------------------
        DropUpload(Wt::WImage *image, Wt::WContainerWidget *parent = 0);
        
        //--------------------------------------------------------------------
        //!  Signal emitted when an upload is too large.
        //--------------------------------------------------------------------
        Wt::Signal<int64_t> & fileTooLarge() { return _fileTooLarge; }
        
        //--------------------------------------------------------------------
        //!  Signal emitted on successful file upload.  getNextFile() may
        //!  be used to get the uploaded files upon reception of this signal.
        //--------------------------------------------------------------------
        Wt::EventSignal<> & uploaded();
        
        //--------------------------------------------------------------------
        //!  Signal emitted when a URI has been received.  getNextURI()
        //!  may be used to get the URI(s) that were received upon reception
        //!  of this signal.
        //--------------------------------------------------------------------
        Wt::EventSignal<> & uriDropped();
        
        //--------------------------------------------------------------------
        //!  If file(s) were uploaded, fills @c uploadedFile with the next
        //!  uploaded file and returns true.  Returns false if there are
        //!  no uploaded files remaining.  Note that it is the caller's
        //!  responsibility to remove the @c uploadedFile from the spool
        //!  when the caller is done with it.  On UNIX, this would be done
        //!  with unlink(uploadedFile.spoolFileName().c_str()).
        //--------------------------------------------------------------------
        bool getNextFile(Wt::Http::UploadedFile & uploadedFile);
        
        //--------------------------------------------------------------------
        //!  If URI(s) were received, fills @c uri with the next received URI
        //!  and returns true.  Returns false if there are no received URIs
        //!  remaining.
        //--------------------------------------------------------------------
        bool getNextURI(std::string & uri);
        
        //--------------------------------------------------------------------
        //!  Accept uploads with the given @c mimeType.  This member may
        //!  be called more than once to add additional MIME types to be
        //!  accepted.
        //--------------------------------------------------------------------
        void acceptDropUploads(const std::string & mimeType);

        //--------------------------------------------------------------------
        //!  Stop accepting uploads with the given @c mimeType.  This member
        //!  may be called more than once to stop accepting uploads of
        //!  more than one MIME type.
        //!  Note that this member doesn't do any regular expression matching.
        //!  If you called acceptDropUploads("image/png"), calling
        //!  stopAcceptDropUploads("image/*") will NOT stop accepting
        //!  uploads with MIME type "image/png".
        //--------------------------------------------------------------------
        void stopAcceptDropUploads(const std::string & mimeType);

        //--------------------------------------------------------------------
        //!  Set the CSS style to be used when droppable items are dragged
        //!  over the widget.  If this member is not called, the default
        //!  style is "dropUploadDragover".
        //--------------------------------------------------------------------
        void setDragoverStyle(const std::string & style);

        //--------------------------------------------------------------------
        //!  Set the CSS style to be used when a droppable item is dropped
        //!  on the widget.  This style will remain until the upload has
        //!  completed.  I find using "fa-spin" from FontAwesome is a
        //!  good @c style choice here.  The default style is
        //!  "dropUploadDrop".
        //--------------------------------------------------------------------
        void setDropStyle(const std::string & style);
        
        friend class UploadResource;
        
      protected:
        UploadResource         _uploadResource;
        Wt::Signal<int64_t>    _fileTooLarge;
        std::set<std::string>  _dropUploadTypes;
        std::string            _dragoverStyle;
        
        void fileTooLarge(int64_t tooLargeSize);
        void filesUploaded();
        void signalURIDropped();
        void CreateJavascriptUtilityFunctions();
        void CreateListeners();
        void CreateDragoverListeners();
        void CreateDropListener();
        void SetJavaScriptMimeTypes();
      };
      
    }  // namespace Gallery

  }  // namespace WWW

}  // namespace Dwm

#endif  // _DWMWWWGALLERYDROPUPLOAD_HH_
